CON

  _clkmode  = xtal1 + pll16x 
  _xinfreq  = 5_000_000

                       
  X_STEP    = 0   '4 parallel port pins INPUTS  0-5
  X_DIR     = 1   '5
  Y_Step    = 2   '6
  Y_Dir     = 3   '7
  Z_Step    = 4   '8
  Z_Dir     = 5   '9
  R_Step    = 6   ' ???
  R_Dir     = 7   ' ???
 
'  coilX1       = 6  'a red
'  coilX2       = 7  'b blue        
'  coilX3       = 8  'c green
'  coilX4       = 9  'd black
  
'  coilY1       = 10 'a red   
'  coilY2       = 11 'b blue  
'  coilY3       = 12 'c green 
'  coilY4       = 13 'd black
  
'  coilZ1       = 14 'a red   
'  coilZ2       = 15 'b blue  
'  coilZ3       = 16 'c green 
'  coilZ4       = 17 'd black
  
  X_EncA    = 18 'quadrature encoder INPUTS  18-25
  X_EncB    = 19
  Y_EncA    = 20
  Y_EncB    = 21
  Z_EncA    = 22
  Z_EncB    = 23
  R_EncA    = 24
  R_EncB    = 25
  
'  Debug_LED = 24    'OUTPUT P24 LED
'  Dis_Mtr   = 25    'Parallel Port Pin16 stop motors from Mach3       INPUT 25
  move_ena  = 26    'OUTPUT Parallel Port Pin 10  enables Mach3  Inverted out!!!!
  PwmClk    = 27    'OUTPUT P27  PWM Clock                       
  
  unused28  = 28  
  unused29  = 29
  unused30  = 30
  unused31  = 31


VAR
  LONG Pos[4]         'encoder var's
  long cogNo

OBJ
   Encoder : "Rotary Encoder"             
   Pwm     : "Pwm"
   driver  : "CNC_new"

PUB START
  outa[move_ena] := 1
  dira[move_ena] := 1
  driver.start( 0, 0, 0, 0)                                             
  PWM.start(27, 50, 200) 
  waitcnt(80_000_000 + cnt)
  Encoder.Start(18, 3, 0, @Pos)
  doCompare(move_ena,-8,8,driver.posBuffer,@Pos,1)
  waitcnt(clkfreq / 1000 + cnt)
  dira[move_ena] := 0
  repeat
    if driver.timestep >  2000   'A = no X travel 
      driver.timestop            'A = no X travel 

' The cog routine watches the driver's position information and the encoder's position information
' and disables the motor if the difference between the two positions exceeds a lower and upper bound
' supplied to the routine.  The original routine assumed a stepper positioning of 400 pulses/rev and
' and encoder quadrature count of 800/rev (half step rate).  This modified routine allows the caller
' to specify a power of two multiplier for both the stepper positioning and the encoder quadrature
' count.  The old half step rate requires a multiplier of 1 (2^1).  Whole step uses a multiplier of
' 2 (2^2).  1/4 step uses a multiplier of 0 (2^0) and 1/16 step uses a multiplier of -2 (2^-2).
' Positive powers are used for a shift instruction to multiply the driver's position.  Negative
' powers are used for a shift instruction to multiply the encoder's position.

PRI doCompare(moveEnaPin, lower, upper, driverBuf, encoderBuf, multiplier)
  cogNo := cognew(@compareEntry,@moveEnaPin) + 1            ' Start assembly routine with parameters
  
DAT
compareEntry           mov      temp1,PAR
                       rdlong   temp2,temp1                 ' Get number of enable pin
                       mov      enaPin,#1
                       shl      enaPin,temp2                ' Convert to a bit mask
                       add      temp1,#4
                       rdlong   lowBnd,temp1                ' Get lower bound of position error
                       add      temp1,#4
                       rdlong   highBnd,temp1               ' Get upper bound of position error
                       add      temp1,#4
                       rdlong   driverPtr,temp1             ' Get address of driver position info
                       add      temp1,#4
                       rdlong   encoderPtr,temp1            ' Get address of encoder position info
                       add      temp1,#4
                       rdlong   temp1,temp1                 ' Get power of 2 for driver:encoder
                       cmps     temp1,#0              wc    '  and see if less than zero
           if_c        mov      driverPwr,#0                ' If so, use absolute value for encoder
           if_c        abs      encoderPwr,temp1            '  shift and zero for driver shift
           if_nc       mov      driverPwr,temp1             ' If not, use positive value for driver
           if_nc       mov      encoderPwr,#0               '  shift and zero for encoder shift
                       mov      OUTA,enaPin
                       mov      DIRA,enaPin                 ' Output a high to enable pin
:startLoop             mov      count,#3
:checkLoop             rdlong   temp1,driverPtr             ' Get driver value and multiply
                       shl      temp1,driverPwr             '  by a specified power of two
                       add      driverPtr,#4
                       rdlong   temp2,encoderPtr            ' Get encoder position and multiply
                       shl      temp2,encoderPwr            '  by a specified power of two
                       add      encoderPtr,#4
                       sub      temp1,temp2                 ' Subtract the two values
                       cmps     temp1,lowBnd          wc
          if_c         mov      OUTA,#0                     ' Disable if difference < lowBnd
                       cmps     highBnd,temp1         wc
          if_c         mov      OUTA,#0                     ' Disable if difference > highBnd
                       djnz     count,#:checkLoop
                       sub      driverPtr,#12
                       sub      encoderPtr,#12
                       jmp      #:startLoop                 ' Start over again with xPos

temp1                  long     0
temp2                  long     0
count                  long     0
enaPin                 long     0                           ' I/O mask for enable pin
lowBnd                 long     0                           ' Lower bound of error difference
highBnd                long     0                           ' Upper bound of error difference
driverPtr              long     0                           ' Address of driver's first value
encoderPtr             long     0                           ' Address of encoder's first value
driverPwr              long     0                           ' Shift count for driver's value
encoderPwr             long     0                           ' Shift count for encoder's value