'' This is an assembly stepper routine that can be called as an object from another object

VAR
  long timeOut, xPos, yPos, zPos, rPos                      ' Current timeOut, xPos, yPos, zPos, rPos
  long cogNo
  
PUB start( xInit, yInit, zInit, rInit)                      ' Start assembly stepper cog
  timeOut := -1
  xPos := xInit
  yPos := yInit                                             ' Initialize the current position
  zPos := zInit
  rPos := rInit
  if (cogNo := cognew(@stepMotor,@timeOut) + 1)
    repeat while timeOut == -1                              ' Wait for cog to finish initialization
    return true
  return false

PUB stop                                                    ' Stop a cog that's been started
  if cogNo
    cogstop(cogNo - 1)

PUB xPosValue
  return xPos

PUB yPosValue
  return yPos

PUB zPosValue
  return zPos
 
PUB rPosValue
  return rPos

PUB timeStep
  if timeOut <> -1 and timeOut < $7FFFFFFF
    timeOut++
  return timeOut

PUB timeStop
  timeOut := -1
  
PUB posBuffer
  return @xPos
  
DAT
stepMotor              mov      xHalfStep,#0                ' Initialize the half step number
                       mov      yHalfStep,#0
                       mov      zHalfStep,#0
                       mov      rHalfStep,#0
                       mov      OUTA,outaMask               ' Initialize I/O registers
                       mov      DIRA,diraMask
                       mov      waitMask,#8                 ' Do a couple of initial steps
:initLoop              mov      tempInA,#%01010101          '  in the forward direction
                       call     #stepOnce
                       mov      temp,cnt                    ' Wait about 10ms between steps
                       add      temp,waitTime
                       waitcnt  temp,#0
                       djnz     waitMask,#:initLoop
                       mov      temp,PAR
                       wrlong   zeroMask,temp               ' Initialize timeOut
                       add      temp,#4
                       rdlong   xPosCog,temp                ' Copy xPos, yPos, zPos, and rPos
                       add      temp,#4                     '  into the local working copies
                       rdlong   yPosCog,temp
                       add      temp,#4
                       rdlong   zPosCog,temp
                       add      temp,#4
                       rdlong   rPosCog,temp

:waitReset             mov      waitMask,#0                 ' Wait loop waits for a change
:wait                  mov      tempInA,INA                 ' Get current input/output status
                       mov      tempOutA,OUTA
                       mov      temp,tempInA                ' See if there's a change in the
                       and      temp,stepMask               '  step pulse inputs from what's
                       cmp      temp,waitMask         wz    '   expected
          if_ne        jmp      #:testBits
                       rdlong   temp,PAR                    ' Get the current timeOut value
                       cmp      temp,minusOne         wz    '  and check if we're stopping
          if_ne        jmp      #:waitReset
                       andn     OUTA,motrMask               ' Turn off the four motors
                       jmp      #:waitReset

:testBits              call     #stepOnce                   ' Do a pass through the stepping code
                       mov      temp,PAR
                       wrlong   zeroMask,temp               ' Clear timeOut
                       add      temp,#4
                       wrlong   xPosCog,temp                ' Copy xPos, yPos, zPos, and rPos into
                       add      temp,#4                     '  respective HUB memory variables
                       wrlong   yPosCog,temp
                       add      temp,#4
                       wrlong   zPosCog,temp
                       add      temp,#4
                       wrlong   rPosCog,temp
                       mov      waitMask,tempInA            ' Next time, wait for a change in the
                       and      waitMask,stepMask           '  step pulse inputs from what was
                       jmp      #:wait                      '   present when we started this

stepOnce               test     tempInA,#%00000001    wc    ' Set carry to step pin
                       test     tempInA,#%00000010    wz    ' Set zero to direction pin
          if_c_and_z   sub      xHalfStep,#1                ' Increment half step if forward
          if_c_and_nz  add      xHalfStep,#1                ' Decrement half step if reverse
          if_c         and      xHalfStep,#7                ' Wraparound half step after 8 steps
          if_c         movs     :getXStep,xHalfStep         ' Get table entry for half step
          if_c         add      :getXStep,#halfStep
          if_c_and_z   sub      xPosCog,#1                  ' While waiting for pipeline delay,
          if_c_and_nz  add      xPosCog,#1                  '  decrement or increment position
:getXStep if_c         mov      temp,0-0                    ' Now get table entry
          if_c         andn     tempOutA,motrMaskX          ' Clear the bits in OUTA for X
          if_c         and      temp,motrMaskX              ' Mask off the stepper bits for X
          if_c         or       tempOutA,temp               ' Accumulate the bits for OUTA

                       test     tempInA,#%00000100    wc    ' Set carry to step pin
                       test     tempInA,#%00001000    wz    ' Set zero to direction pin
          if_c_and_z   add      yHalfStep,#1                ' Increment half step if forward
          if_c_and_nz  sub      yHalfStep,#1                ' Decrement half step if reverse
          if_c         and      yHalfStep,#7                ' Wraparound half step after 8 steps
          if_c         movs     :getYStep,yHalfStep         ' Get table entry for half step
          if_c         add      :getYStep,#halfStep
          if_c_and_z   add      yPosCog,#1                  ' While waiting for pipeline delay,
          if_c_and_nz  sub      yPosCog,#1                  '  decrement or increment position
:getYStep if_c         mov      temp,0-0                    ' Now get table entry
          if_c         andn     tempOutA,motrMaskY          ' Clear the bits in OUTA for Y
          if_c         and      temp,motrMaskY              ' Mask off the stepper bits for Y
          if_c         or       tempOutA,temp               ' Accumulate the bits for OUTA

                       test     tempInA,#%00010000    wc    ' Set carry to step pin
                       test     tempInA,#%00100000    wz    ' Set zero to direction pin
          if_c_and_z   sub      zHalfStep,#1                ' Increment half step if forward **rev
          if_c_and_nz  add      zHalfStep,#1                ' Decrement half step if reverse **rev
          if_c         and      zHalfStep,#7                ' Wraparound half step after 8 steps
          if_c         movs     :getZStep,zHalfStep         ' Get table entry for half step
          if_c         add      :getZStep,#halfStep
          if_c_and_z   sub      zPosCog,#1                  ' While waiting for pipeline delay,
          if_c_and_nz  add      zPosCog,#1                  '  decrement or increment position
:getZStep if_c         mov      temp,0-0                    ' Now get table entry
          if_c         andn     tempOutA,motrMaskZ          ' Clear the bits in OUTA for Z
          if_c         and      temp,motrMaskZ              ' Mask off the stepper bits for Z
          if_c         or       tempOutA,temp               ' Accumulate the bits for OUTA

                       test     tempInA,#%01000000    wc    ' Set carry to step pin
                       test     tempInA,#%10000000    wz    ' Set zero to direction pin
          if_c_and_z   sub      rHalfStep,#1                ' Increment half step if forward **rev
          if_c_and_nz  add      rHalfStep,#1                ' Decrement half step if reverse **rev
          if_c         and      rHalfStep,#7                ' Wraparound half step after 8 steps
          if_c         movs     :getRStep,rHalfStep         ' Get table entry for half step
          if_c         add      :getRStep,#halfStep
          if_c_and_z   sub      rPosCog,#1                  ' While waiting for pipeline delay,
          if_c_and_nz  add      rPosCog,#1                  '  decrement or increment position
:getRStep if_c         mov      temp,0-0                    ' Now get table entry
          if_c         andn     tempOutA,motrMaskR          ' Clear the bits in OUTA for R
          if_c         and      temp,motrMaskR              ' Mask off the stepper bits for R
          if_c         or       tempOutA,temp               ' Accumulate the bits for OUTA

                       mov      OUTA,tempOutA               ' Finally update output register
stepOnce_ret           ret

'                                PM RRZZYYXX            RRZZYYXX
'                                wv EEEEEEEE            dsdsdsds
'                                mE nnnnnnnn            itititit
'                                Cn cccccccc            rererere
'                                la BABABABA            |p|p|p|p
halfStep               long     %00_00000000_0000000000_00000000     ' Half step values
                       long     %00_00000000_0000000000_00000000
                       long     %00_00000000_0000000000_00000000
                       long     %00_00000000_0000000000_00000000
                       long     %00_00000000_0000000000_00000000
                       long     %00_00000000_0000000000_00000000
                       long     %00_00000000_0000000000_00000000
                       long     %00_00000000_0000000000_00000000
motrMaskX              long     %00_00000000_0000000000_00000000     ' Mask for X motor
motrMaskY              long     %00_00000000_0000000000_00000000     ' Mask for Y motor
motrMaskZ              long     %00_00000000_0000000000_00000000     ' Mask for Z motor
motrMaskR              long     %00_00000000_0000000000_00000000     ' Mask for R motor
motrMask               long     %00_00000000_0000000000_00000000     ' Mask for all four motors
zeroMask               long     %00_00000000_0000000000_00000000     ' All zero bits
stepMask               long     %00_00000000_0000000000_01010101     ' Mask for step pins
diraMask               long     %00_00000000_0000000000_00000000     ' I/O direction for motors
outaMask               long     %00_00000000_0000000000_00000000     ' Initial OUTA for motors
minusOne               long     -1
waitTime               long     10 * (80_000_000 / 1000)             ' 10ms delay with 80MHz clock
xPosCog                long     0
xHalfStep              long     0
yPosCog                long     0
yHalfStep              long     0
zPosCog                long     0
zHalfStep              long     0
rPosCog                long     0
rHalfStep              long     0
tempInA                long     0
temp                   long     0
tempOutA               long     0
waitMask               long     0